@@ -1,22 +1,24 @@
-import os
+from __future__ import annotations
 
+import os
 from typing_extensions import Final
 
 PYTHON2_VERSION: Final = (2, 7)
-PYTHON3_VERSION: Final = (3, 6)
+
+# Earliest fully supported Python 3.x version. Used as the default Python
+# version in tests. Mypy wheels should be built starting with this version,
+# and CI tests should be run on this version (and later versions).
+PYTHON3_VERSION: Final = (3, 7)
+
+# Earliest Python 3.x version supported via --python-version 3.x. To run
+# mypy, at least version PYTHON3_VERSION is needed.
 PYTHON3_VERSION_MIN: Final = (3, 4)
+
 CACHE_DIR: Final = ".mypy_cache"
 CONFIG_FILE: Final = ["mypy.ini", ".mypy.ini"]
-PYPROJECT_CONFIG_FILES: Final = [
-    "pyproject.toml",
-]
-SHARED_CONFIG_FILES: Final = [
-    "setup.cfg",
-]
-USER_CONFIG_FILES: Final = [
-    "~/.config/mypy/config",
-    "~/.mypy.ini",
-]
+PYPROJECT_CONFIG_FILES: Final = ["pyproject.toml"]
+SHARED_CONFIG_FILES: Final = ["setup.cfg"]
+USER_CONFIG_FILES: Final = ["~/.config/mypy/config", "~/.mypy.ini"]
 if os.environ.get("XDG_CONFIG_HOME"):
     USER_CONFIG_FILES.insert(0, os.path.join(os.environ["XDG_CONFIG_HOME"], "mypy/config"))
 
@@ -42,5 +44,5 @@ REPORTER_NAMES: Final = [
 ]
 
 # Threshold after which we sometimes filter out most errors to avoid very
-# verbose output
-MANY_ERRORS_THRESHOLD: Final = 200
+# verbose output. The default is to show all errors.
+MANY_ERRORS_THRESHOLD: Final = -1
